<?php
/*
 * TDTIS - Copyright 2006 Alternance-Soft
 * Contributeur : Jrme Schell, Aot 2006 
 *
 * contact@alternancesoft.com
 *
 * Ce logiciel est un programme informatique servant  la
 * dmatrialisation de l'administration. 
 *
 * Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilit au code source et des droits de copie,
 * de modification et de redistribution accords par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limite.  Pour les mmes raisons,
 * seule une responsabilit restreinte pse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concdants successifs.
 *
 * A cet gard  l'attention de l'utilisateur est attire sur les risques
 * associs au chargement,   l'utilisation,   la modification et/ou au
 * dveloppement et  la reproduction du logiciel par l'utilisateur tant 
 * donn sa spcificit de logiciel libre, qui peut le rendre complexe  
 * manipuler et qui le rserve donc  des dveloppeurs et des professionnels
 * avertis possdant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invits  charger  et  tester  l'adquation  du
 * logiciel  leurs besoins dans des conditions permettant d'assurer la
 * scurit de leurs systmes et ou de leurs donnes et, plus gnralement, 
 *  l'utiliser et l'exploiter dans les mmes conditions de scurit. 
 *
 * Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
 * pris connaissance de la licence CeCILL, et que vous en avez accept les
 * termes.
*/
?>
<?php
/**
 * \class Group Group.class.php
 * \brief Classe de gestion des groupes de collectivits
 * \author Jrme Schell <j.schell@alternancesoft.com>
 * \date 12.01.2007
 * 
 *
 * Cette classe permet de grer les diffrents groupes de collectivits.
 * Les mthodes de bases sont hrites de DataObject.
 *
 * Modifications :
 * Auteur   Date       Commentaire
 *
 */

require_once("DataObject.class.php");
require_once("Module.class.php");
require_once("Siren.class.php");

class Group extends DataObject {
  protected $objectName = "authority_groups";
  protected $prettyName = "Groupes";

  protected $name;
  protected $status;

  private $recordSiren = false;
  private $sirenList = array();

  protected $dbFields = array(
							  "name" => array( "descr" => "Nom", "type" => "isString", "mandatory" => true, "unique" => true ),
							  "status" => array( "descr" => "tat", "type" => "isInt", "mandatory" => true)
							  );


  /**
   * \brief Constructeur d'un groupe de collectivit
   * \param id integer Numro d'id d'un groupe de collectivit existante avec lequel initialiser l'objet
   */
  public function __construct($id = false) {
    parent::__construct($id);
	if ($id) {
	  $this->init();
	}
  }

  /**
   * \brief Mthode d'initialisation d'un groupe de collectivit depuis la base de donnes
   * \return true si succs, false sinon
  */
  public function init() {
	return (parent::init() && $this->initSiren());
  }

  /**
   * \brief Mthode qui renvoie la liste des numros de SIREN autoriss pour ce groupe
   */
  public function getAuthorizedSiren() {
	return $this->sirenList;
  }

  /**
   * \brief Mthode permettant de dterminer si un numro de SIREN est autoris pour le groupe
   * \param $num entier : Numro de SIREN  tester
   * \return True si le SIREN est autoris, False sinon
   */
  public function isAuthorizedSiren($num) {
	if (array_search($num, $this->sirenList) === false) {
	  return false;
	} else {
	  return true;
	}
  }

  /**
   * \brief Mthode d'importation du fichier contenant la liste des SIREN autoriss pour le groupe
   * \param $file chane : chemin du fichier  importer
   * \return True si succs, False sinon
   */
  public function importSiren($file) {
	$this->resetSirenList();

	if (! $handle = fopen($file, "r")) {
	  $this->errorMsg = "Erreur lors de l'ouverture du fichier.";
	  return false;
	}

        $theSiren  = new Siren();
	while ($content = fgets($handle)) {
	  $content = trim($content);

	  if (VERIFICATION_SIREN)
	  {		
              if (! $theSiren->isValid($content)) {
                  $this->errorMsg="erreur lors de l'analyse du numro SIREN ($content).";
                  return false;
              }  
	  }
	  if (! empty($content) && strlen($content) <= 9) {
		$this->sirenList[] = $content;
	  }
	}

	$this->recordSiren = true;

	return true;
  }


  /**
   * \brief Mthode de remise  zro des permissions sur les modules
  */
  public function resetSirenList() {
	$this->sirenList = array();
  }

  /**
   * \brief Mthode d'initialisation de la liste des Siren autoriss pour ce groupe
  */
  public function initSiren() {
	if (isset($this->id)) {
	  $sql = "SELECT siren FROM authority_group_siren " .
	  			" WHERE authority_group_id={$this->id}" .
	  			" ORDER BY siren";

	  $result = $this->db->select($sql);

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  $this->sirenList[] = $row["siren"];
		}
	  } else {
		return false;
	  }
	} else {
	  return false;
	}

	return true;
  }

  /**
   * \brief Mthode permettant de savoir si une collectivit est active
   * \return true si active, false sinon
  */
  public function isActive() {
    return ($this->status == 1);
  }

  /**
   * \brief Mthode d'enregistrement d'un groupe dans la base de donnes
   * \param $validate boolen (optionnel) Prcise si la validation de l'entit doit avoir lieu (true par dfaut)
   * \return true si succs, false sinon
   */
  public function save($validate = true,$bouchon_4_strict_standard = true) {
    if (! ($sql = parent::save($validate, true))) {
	  return false;
	}
  	 
  	//echo $sql;
    //exit();
    if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

    if (! $this->db->exec($sql)) {
      $this->errorMsg = "Erreur lors de la sauvegarde du groupe.";
	  $this->db->rollback();
      return false;
    }

	if ($this->recordSiren) {    
	  //! Traitement des siren associs au groupe
	  $sql = "DELETE FROM authority_group_siren WHERE authority_group_id=" . $this->id;
    
	  if (! $this->db->exec($sql)) {
		$this->errorMsg = "Erreur lors de la rinitialisation des siren associs au module.";
		$this->db->rollback();
		return false;
	  }

	  if (count($this->sirenList) > 0) {
		reset($this->sirenList);
		foreach ($this->sirenList as $siren) {
		  $sql = "INSERT INTO authority_group_siren (authority_group_id, siren) VALUES(" . $this->id . ", '" . addslashes($siren) . "')";
		  if (! $this->db->exec($sql)) {
			$this->errorMsg = "Erreur lors de la sauvegarde des siren du groupe.";
			$this->db->rollback();
			return false;
		  }
		}
	  }
	}

    if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

    return true;
  }

  /**
   * \brief Mthode permettant de supprimer un groupe de la base de donnes
   * \param $id integer (optionnel) Numro d'identifiant du groupe, si non spcifi, groupe en cours
   * \return true si succs, false sinon
  */
  public function delete($id = false) {
    // Efface l'entit spcifie par $id ou alors l'entit courante si pas d'id
    if (! $id) {
      if (isset($this->id) && ! empty($this->id)) {
		$id = $this->id;
      } else {
		$this->errorMsg = "Pas d'identifiant pour l'entit a supprimer";
		return false;
      }
    }

	if (! $this->isEmpty($id)) {
	  $this->errorMsg = "Le groupe n'est pas vide. Suppression impossible.";
	  return false;
	}

	if (! $this->db->begin()) {
      $this->errorMsg = "Erreur lors de l'initialisation de la transaction.";
      return false;
	}

	$sql = "DELETE FROM authority_group_siren WHERE authority_group_id=" . $id;

    if (! $this->db->exec($sql)) {
	  $this->errorMsg = "Erreur lors de la suppression des siren autoriss pour le groupe.";
	  $this->db->rollback();
	  return false;
    }

	if (! parent::delete($id)) {
	  $this->db->rollback();
      return false;
	}

	if (! $this->db->commit()) {
      $this->errorMsg = "Erreur lors de la validation de la transaction.";
	  $this->db->rollback();
      return false;
	}

	return true;
  }

  /**
   * \brief Mthode d'obtention de la liste des groupes et tous leurs attributs
   * \param $cond (optionnel) chane Chane contenant les conditions (SQL)  appliquer  la fin de la requte BDD
   * \return tableau des groupes
  */
  public function getGroupsList($cond = "") {
	if (! $this->pagerInit('authority_groups.id, authority_groups.name, authority_groups.status', 'authority_groups', $cond, 'authority_groups.name', null, null, 'asc')) {
	  return false;
	}

    return $this->data;
  }



  /**********************/
  /* Mthodes statiques */
  /**********************/

  /**
   * \brief Mthode qui dtermine si un groupe comporte des collectivits
   * \param $id integer Identifiant du groupe concern
   * \return True si le groupe est vide, false sinon
   *
  */
  public static function isEmpty($id) {
    $sql = "SELECT authorities.id FROM authorities WHERE authority_group_id=" . $id;

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

    if (! $result->isError()) {
      if ($result->num_row() > 0) {
		return false;
	  } else {
		return true;
	  }
    } else {
	  return false;
	}
  }

  /**
   * \brief Mthode d'obtention d'une liste de groupe
   * \return Tableau de groupe
   *
   * Cette mthode retourne un tableau dont les clefs sont les identifiants
   * des groupe et le contenu de la case est le nom du groupe
  */
  public static function getGroupsIdName() {
    $sql = "SELECT authority_groups.id, authority_groups.name FROM authority_groups ORDER BY authority_groups.name ASC";

    $db =DatabasePool::getInstance();

    $result = $db->select($sql);

    if (! $result->isError()) {
	  $tabGroups = array();

	  while ($row = $result->get_next_row()) {
		$tabGroups[$row["id"]] = $row["name"];
	  }
    } else {
	  return false;
	}
   return $tabGroups;
  }
  /**
   * @brief obtenir la list de group
   * @return String array liste du group
   */
  public static function getGroupList()
  {
	  $sql = "SELECT , authority_districts.name FROM authority_districts LEFT JOIN authority_departments ON authority_districts.authority_department_id=authority_departments.id WHERE authority_departments.code='" . $dept . "'";

	  $db =DatabasePool::getInstance();

	  $result = $db->select($sql);

	  if (! $result->isError()) {
		while ($row = $result->get_next_row()) {
		  $districts[$row["code"]] = $row["name"];
		}
	  }
  }
}
?>
